#include "h/types.h"
#include "h/interface.h"
#include "h/mapper.h"
#include "../nes/h/nesGeneral.h"
#include "../nes/h/ppu.h"
#include "../nes/h/cpu.h"
#include "h/emuFileHandling.h"

#ifdef staticMapper
	#ifndef notExternC
		extern "C"
		{
	#endif
			PDLLInfo	 nsf_Load_DLL (int VersionRequired);
			PDLLInfo	iNes_Load_DLL (int VersionRequired);
			PDLLInfo	unif_Load_DLL (int VersionRequired);
			PDLLInfo	 fds_Load_DLL (int VersionRequired);

			void	 nsf_Unload_DLL ();
			void	iNes_Unload_DLL ();
			void	unif_Unload_DLL ();
			void	 fds_Unload_DLL ();
	#ifndef notExternC
		};
	#endif
#endif

namespace Reminesce
{

//extern u8 *myTiles_;

void setMirror(u8 Type)
{
	if (header.fourScreenMirroring)
		Type = fScreen;
	switch (Type)
	{
	case vert:
		pntBank[0] = &nameTable[0];
		pntBank[1] = &nameTable[0x400];
		pntBank[2] = &nameTable[0];
		pntBank[3] = &nameTable[0x400];
		break;
	case horz:
		pntBank[0] = &nameTable[0];
		pntBank[1] = &nameTable[0];
		pntBank[2] = &nameTable[0x400];
		pntBank[3] = &nameTable[0x400];
		break;
	case fScreen:
		pntBank[0] = &nameTable[0];
		pntBank[1] = &nameTable[0x400];
		pntBank[2] = &nameTable[0x800];
		pntBank[3] = &nameTable[0xC00];
		break;
	case single0:
		pntBank[0] = &nameTable[0];
		pntBank[1] = &nameTable[0];
		pntBank[2] = &nameTable[0];
		pntBank[3] = &nameTable[0];
		break;
	case single1:
		pntBank[0] = &nameTable[0x400];
		pntBank[1] = &nameTable[0x400];
		pntBank[2] = &nameTable[0x400];
		pntBank[3] = &nameTable[0x400];
		break;
	case single2:
		pntBank[0] = &nameTable[0x800];
		pntBank[1] = &nameTable[0x800];
		pntBank[2] = &nameTable[0x800];
		pntBank[3] = &nameTable[0x800];
		break;
	case single3:
		pntBank[0] = &nameTable[0xC00];
		pntBank[1] = &nameTable[0xC00];
		pntBank[2] = &nameTable[0xC00];
		pntBank[3] = &nameTable[0xC00];
		break;
	}
}

void setPrgBanks(int prgPage, int prgK, int numofK)
{
	numofK	/= 2;				//2k at a time
	prgPage /= 0x800;
	prgK &= header.prg8;
	prgK *= 1024;

	for (int i=0; i<numofK; i++)
	{
		pbankWrite[prgPage + i] = deadWrite;
		pbankRead [prgPage + i] = &prgromPtr[(i*0x800) + prgK];
	}
}

void setPrgBanks2(int prgPage, int prgK, int numofK)
{
	numofK	/= 2;				//2k at a time
	prgPage /= 0x800;
	prgK &= header.prg8;
	prgK *= 1024;

	for (int i=0; i<numofK; i++)
	{
		pbankWrite[prgPage + i] =
		pbankRead [prgPage + i] = &wram[(i*0x800) + prgK];
	}
}

void setChrBanks(int chrPage, int chrK, int numofK)
{
	if (((chrK + numofK) > 8))
		numofK=numofK;

	chrPage /= 1024;
	chrK    &= header.chr1;
	chrK    *= 1024;
	for (int i=0; i<numofK; i++)
	{
		pChrWrite[chrPage+i] = deadWrite;
		pChrRead [chrPage+i] = &chrromPtr[i*1024 + chrK];
//		pMyTile  [chrPage+i] = &myTiles_[i*4096 + chrK*4];
	}
}

void setChrBanks2(int chrPage, int chrK, int numofK)
{
	if ((numofK - chrK) > 16)
		return;

	chrPage /= 1024;
	chrK    *= 1024;
	for (int i=0; i<numofK; i++)
	{
		pChrWrite[chrPage+i] = 
		pChrRead [chrPage+i] = &chrRam[i*1024 + chrK];
	}
}



///////////////////////////////////////////////////////////////////////////////

typedef PDLLInfo	(*nestenLoad_DLL)	(int VersionRequired);
typedef void		(*nestenUnload_DLL)	();

extern nestenLoad_DLL		nestenMapLoad;
extern nestenUnload_DLL		nestenMapUnload;

#ifndef staticMapper
	#ifdef WIN32
		#include <windows.h>
	#endif

	HINSTANCE inesDll;

	char loadNestenMap()
	{
	#if _DEBUG
		const char libPath[] = "MAPPERS/INES.dll";
	#else
		const char libPath[] = "Mappers/INES.dll";
	#endif

		inesDll			= LoadLibrary(libPath);
		if (!inesDll)
		{
			emuMsg("couldn't LoadLibrary");
			return 1;
		}

		nestenMapLoad	= (nestenLoad_DLL)		GetProcAddress(inesDll, "Load_DLL");
		nestenMapUnload	= (nestenUnload_DLL)	GetProcAddress(inesDll, "Unload_DLL");

		if (!nestenMapLoad || !nestenMapUnload)
		{
			emuMsg("couldn't GetProcAddress");
			return 2;
		}

		nestenDll		= nestenMapLoad(CurrentMapperInterface);
		if (!nestenDll)
		{
			emuMsg("couldn't load Mapper Interface (nestenMapLoad())");
			return 3;
		}
		
		return 0;
	}

	void unloadNestenMap()
	{
		if (mapInfo)
			mapInfo->UnloadMapper();
		mapInfo = 0;

		if (inesDll)
		{
			if (nestenMapUnload)
				nestenMapUnload();

			FreeLibrary(inesDll);
		}
		inesDll = 0;
	}
#else

	char loadNestenMap()
	{
		nestenMapLoad		= iNes_Load_DLL;
		nestenMapUnload		= iNes_Unload_DLL;
		nestenDll			= nestenMapLoad(CurrentMapperInterface);
		if (!nestenDll)
		{
			emuMsg("couldn't load Mapper Interface (nestenMapLoad())");
			return 3;
		}
		
		return 0;
	}

	void unloadNestenMap()
	{
		if (mapInfo)
			mapInfo->UnloadMapper();
		mapInfo = 0;


		if (nestenMapUnload)
			nestenMapUnload();
	}
#endif

}
