
// Common headers used by Shay Green's libraries

#ifndef BLARGG_COMMON_H
#define BLARGG_COMMON_H

// Allow prefix configuration file *which can re-include blargg_common.h*
// (probably indirectly).
#ifdef HAVE_CONFIG_H
	#undef HAVE_CONFIG_H
	#undef BLARGG_COMMON_H
	#include "config.h"
	#define BLARGG_COMMON_H
#endif

// check for boost availability
#include "boost/config.hpp"
#ifndef BOOST
	#define BOOST boost
#endif

// BLARGG_BIG_ENDIAN and BLARGG_LITTLE_ENDIAN
#if !defined (BLARGG_BIG_ENDIAN) && !defined (BLARGG_LITTLE_ENDIAN)
	#if defined (__powerc) || defined (macintosh)
		#define BLARGG_BIG_ENDIAN 1
	
	#elif defined (_MSC_VER) && defined (_M_IX86)
		#define BLARGG_LITTLE_ENDIAN 1
	
	#endif
#endif

// BLARGG_CPU_*
#if !defined (BLARGG_CPU_POWERPC) && !defined (BLARGG_CPU_X86)
	#if defined (__powerc)
		#define BLARGG_CPU_POWERPC 1
	
	#elif defined (_MSC_VER) && defined (_M_IX86)
		#define BLARGG_CPU_X86 1
	
	#endif
#endif

// BLARGG_NONPORTABLE (allow use of nonportable optimizations/features)
#ifndef BLARGG_NONPORTABLE
	#define BLARGG_NONPORTABLE 0
#endif
#ifdef BLARGG_MOST_PORTABLE
	#error "BLARGG_MOST_PORTABLE has been removed; see BLARGG_NONPORTABLE."
#endif

// BLARGG_DEBUG
#ifndef BLARGG_DEBUG
	#ifdef NDEBUG
		#define BLARGG_DEBUG 0
	#else
		#define BLARGG_DEBUG 1
	#endif
#endif

// BLARGG_ENABLE_OPTIMIZER include filename
#ifndef BLARGG_ENABLE_OPTIMIZER
	#define BLARGG_ENABLE_OPTIMIZER "blargg_common.h"
#endif

// BOOST_STATIC_ASSERT( expr )
#include "boost/static_assert.hpp"

// blargg_err_t (NULL on success, otherwise error string)
typedef const char* blargg_err_t;
#define blargg_success 0

// If expr returns error string, return it from current function, otherwise continue.
#define BLARGG_RETURN_ERR( expr ) do {                          \
		blargg_err_t blargg_return_err_ = (expr);               \
		if ( blargg_return_err_ ) return blargg_return_err_;    \
	} while ( 0 )

// BOOST::uint8_t, BOOST::int16_t, etc.
#include "boost/cstdint.hpp"

// bool, true, false
#ifndef BLARGG_COMPILER_HAS_BOOL
	#if !BOOST_MINIMAL
		#define BLARGG_COMPILER_HAS_BOOL 1
	
	#elif defined (__MWERKS__)
		#if !__option(bool)
			#define BLARGG_COMPILER_HAS_BOOL 0
		#endif
	
	#elif defined (_MSC_VER)
		#if _MSC_VER < 1100
			#define BLARGG_COMPILER_HAS_BOOL 0
		#endif
	
	#elif defined (__MINGW32__)
		#define BLARGG_COMPILER_HAS_BOOL 1
	
	#elif __cplusplus < 199711
		#define BLARGG_COMPILER_HAS_BOOL 0
	
	#endif
#endif
#ifndef BLARGG_COMPILER_HAS_BOOL
	#define BLARGG_COMPILER_HAS_BOOL 1
#endif

#if !BLARGG_COMPILER_HAS_BOOL
	typedef int bool;
	const bool true  = 1;
	const bool false = 0;
#endif

#include <stddef.h>
#include <assert.h>

#endif

