#include <stdio.h>
#include <string.h>

#include "../emu/h/emuMemory.h"
#include "../emu/h/types.h"
#include "../emu/h/emuFileHandling.h"
#include "h/nesGeneral.h"
#include "h/nes.h"
#include "h/cpu.h"
#include "h/ppu.h"
#include "../emu/h/interface.h"
#include "../emu/h/mapper.h"
#include "blargg/Nes_Apu.h"
#include "blargg/Blip_Buffer.h"
#include "h/apu.h"

namespace Reminesce
{

u8 openBanks[openBankSize], deadWrite[0x800];
u8 *pbankRead[32], *pbankWrite[32];

u8 *ram, *wram;
u8 brunNes, imageLoaded=0, nesMode=0;

static u8 dummyArray [] = { 0x12, 0x34, 0x56 };
void deleteTiles();

void HardResetNes()
{
#if defined(LITTLE_ENDIAN) && defined (CAN_UNALIGN_READ)
	if  (
		((* ((short*) &dummyArray[0])) != 0x3412) &&
		((* ((short*) &dummyArray[1])) != 0x5634)
		)
	{
		emuMsg("This has been compiled for little endian cpus which can handle unaligned reads. remove the CAN_UNALIGN_READ to fix this");
		imageLoaded = 0;
		return;
	}
#endif

#ifdef BIG_ENDIAN
	if  ( (* ((short*) &dummyArray[0])) != 0x1234)
	{
		emuMsg("This has been compiled for big endian which this is not. Remove BIG_ENDIAN");
		imageLoaded = 0;
		return;
	}
#else
	if  ( (* ((short*) &dummyArray[0])) != 0x3412)
	{
		emuMsg("This has been compiled for little endian which this is not. Remove compile with BIG_ENDIAN");
		imageLoaded = 0;
		return;
	}
#endif

	if (mapInfo)
	{
		if (mapInfo->UnloadMapper)
			mapInfo->UnloadMapper();
		mapInfo = 0;
	}
	memset(&cpu,		0, sizeof(cpu));
	memset(&ppu,		0, sizeof(ppu));
	memset(ram,			0, ramSize);
	memset(wram,		0, wramSize);
	memset(sprRam,		0, sizeof(sprRam));
	memset(chrRam,		0, chrramSize);
	memset(nameTable,	0, NtSize);
	memset(pal,			0, sizeof(pal));
	setChrBanks2(0, 0, 16);
	if (loadMapperNumber(header.mapNumber))
	{
		imageLoaded = 0;
		return;
	}

	nesMode |= resetLoop;
	mapInfo->InitMapper(&mp, 1);
	initApu();
	ResetNes();
}

int  myReadFunc (int bank, int where);
void myWriteFunc(int bank, int where, int what);

void ResetNes()
{
	if (!imageLoaded)								//Check pointer too see if file loaded
		return;

	if (mapInfo)
		if (mapInfo->UnloadMapper)
			mapInfo->UnloadMapper();

	for (int n=0; n<0x10; n++)
	{
		mp.SetWriteHandler(n, myWriteFunc);
		mp.SetReadHandler (n, myReadFunc);
	}

	mapInfo->InitMapper(&mp, 0);
	cpuCycles = cpuTargetCycles = 0;				//should clear on new frame?
	memloc.si_ = temp.si_ = 0;
	cpu.s  -= 3;
	cpu.I   = cI;
	cpu.pc  = pRead16(RESET);

	resetTheApu();
}

void SetpBanks()
{
	int i;
	pbankRead [0] = pbankRead [1] = pbankRead [2] = pbankRead [3] =
	pbankWrite[0] = pbankWrite[1] = pbankWrite[2] = pbankWrite[3] = ram;
	for (i=0; i<8; i++)
	{
		pbankWrite[8+i] = deadWrite;
		pbankRead [8+i] = &openBanks[i * 0x800];
	}
	for (i=0; i < 4; i++)
	{
		pbankRead [4+i] = deadWrite;
		pbankWrite[4+i] = deadWrite;
	}
}

void initBanks()
{
	for (int i=0; i<0x4000; i++)
		openBanks[i] = (0x4000 + i)>>8;
}

int nesInit()
{
	brunNes = 0;
	initMp();

	if (loadNestenMap())
		return 1;

	generateRevBits();									//for PPU
	AttributeOffsetGenerator();
	initBanks();
	SetpBanks();
	return 0;
}

void nesClean()
{
	deleteTiles();
	uninitApu();
	if (imageLoaded)
	{
		deletePrgChrMemory();
	}

	unloadNestenMap();
}
}
